﻿//-----------------------------------------------------------------------------
// Game1.cs
//
// Kyoung Shin Park @ Dankook University (4/4/2011)
//-----------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using XNAGeometryObject;

namespace XNATransformation
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // input
        GamePadState currentGamePadState;
        GamePadState prevGamePadState;
        KeyboardState currentKeyboardState;
        KeyboardState prevKeyboardState;

        // Store a list of primitive models, plus which one is currently selected.
        List<GeometricPrimitive> primitives = new List<GeometricPrimitive>();
        float angle = 0.0f;
        float zoom = 1.0f;

        // Are we rendering in wireframe mode?
        bool isWireframe = true;
        bool isAnimation = true; 

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            primitives.Add(new CubePrimitive(GraphicsDevice));
            primitives.Add(new SpherePrimitive(GraphicsDevice));
            primitives.Add(new CylinderPrimitive(GraphicsDevice));
            primitives.Add(new TorusPrimitive(GraphicsDevice));
            //primitives.Add(new TeapotPrimitive(GraphicsDevice));
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            HandleInput();

            base.Update(gameTime);
        }

        #region HandleInput
        protected void HandleInput()
        {
            prevGamePadState = currentGamePadState;
            currentGamePadState = GamePad.GetState(PlayerIndex.One);
            prevKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState(PlayerIndex.One);

            // Check for exit.
            if (IsPressed(Keys.Escape, Buttons.Back))
            {
                Exit();
            }

            // Toggle animation?
            if (IsPressed(Keys.A, Buttons.A))
            {
                isAnimation = !isAnimation;
            }

            // Toggle wireframe?
            if (IsPressed(Keys.W, Buttons.B))
            {
                isWireframe = !isWireframe;
            }

            // Change Y angle?
            if (IsPressed(Keys.X, Buttons.X))
            {
                angle += 5.0f;
            }

            // Change Translate?
            if (IsPressed(Keys.Y, Buttons.Y))
            {
                zoom -= 0.1f;
            }
        }

        /// <summary>
        /// Checks whether the specified key or button has been pressed.
        /// </summary>
        bool IsPressed(Keys key, Buttons button)
        {
            return (currentKeyboardState.IsKeyDown(key) &&
                    prevKeyboardState.IsKeyUp(key)) ||
                   (currentGamePadState.IsButtonDown(button) &&
                    prevGamePadState.IsButtonUp(button));
        }
        #endregion

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            RenderState renderState = GraphicsDevice.RenderState;
            if (isWireframe)
            {
                renderState.FillMode = FillMode.WireFrame;
                renderState.CullMode = CullMode.None;
            }
            else
            {
                renderState.FillMode = FillMode.Solid;
                renderState.CullMode = CullMode.CullCounterClockwiseFace;
            }

            // Create camera matrices, making the object spin.
            float time = (float)gameTime.TotalGameTime.TotalSeconds;
            if (isAnimation) angle = time * 30.0f;

            Vector3 cameraPosition = new Vector3(0, 0, 3.5f);
            float aspect = GraphicsDevice.Viewport.AspectRatio;

            Matrix world = Matrix.Identity; 
            //Matrix world = Matrix.CreateRotationY(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom);
            Matrix view = Matrix.CreateLookAt(cameraPosition, Vector3.Zero, Vector3.Up);
            Matrix projection = Matrix.CreatePerspectiveFieldOfView(1, aspect, 1, 10);

            // Draw the transformed cube, sphere, cylinder, torus primitives.
            world = Matrix.CreateRotationY(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom) * Matrix.CreateTranslation(-1.0f, 0.0f, 0.0f);
            primitives[0].Draw(world, view, projection, Color.Blue);

            world = Matrix.CreateRotationY(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom) * Matrix.CreateScale(1.5f, 1.5f, 1.5f) * Matrix.CreateTranslation(1.0f, 0.0f, 0.0f);
            primitives[1].Draw(world, view, projection, Color.Red);

            world = Matrix.CreateRotationX(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom) * Matrix.CreateTranslation(0.0f, 0.0f, -1.0f);
            primitives[2].Draw(world, view, projection, Color.Green);

            world = Matrix.CreateRotationX(MathHelper.ToRadians(angle)) * Matrix.CreateTranslation(0, 0, zoom) * Matrix.CreateTranslation(0.0f, 0.0f, 1.0f);
            primitives[3].Draw(world, view, projection, Color.Magenta);

            base.Draw(gameTime);
        }
    }
}
